package com.tofly.watermeter.custom.service.meterBusiness.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.util.DateUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ImportTurnOver;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ImportTurnOverDto;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.TurnOverDto;
import com.tofly.watermeter.custom.mapper.meterBusiness.TurnOverMapper;
import com.tofly.watermeter.custom.service.meterBusiness.TurnOverService;
import com.tofly.watermeter.custom.service.meterBusiness.export.ExportCommonCodeService;
import com.tofly.watermeter.entity.Bwrksbdatail;
import com.tofly.watermeter.entity.Bwsbyjgl;
import com.tofly.watermeter.service.BwrksbdatailService;
import com.tofly.watermeter.service.BwsbyjglService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class TurnOverServiceImpl implements TurnOverService {

    @Autowired
    private TurnOverMapper turnOverMapper;
    @Autowired
    private BwrksbdatailService bwrksbdatailService;
    @Autowired
    private ExportCommonCodeService exportCommonCodeService;
    @Autowired
    private BwsbyjglService bwsbyjglService;

    @Override
    public Page<Map> canTurnOver(Page page, TurnOverDto dto) {
        return turnOverMapper.canTurnOver(page, dto);
    }

    @Override
    public List<Map> count(HttpServletResponse response, TurnOverDto dto) {
        List<Map> count = turnOverMapper.count(dto);

        if (dto.getExport() == 1) {
            exportCommonCodeService.exportCode(response, "turnOverCountService", count);
        }
        return count;
    }

    @Override
    public Page<Map> queryInfo(HttpServletResponse response, Page page, TurnOverDto dto) {
        if (dto.getExport() == 1) {
            exportCommonCodeService.exportCode(response, "turnOverQueryInfoService", turnOverMapper.queryInfo(dto));
        }
        return turnOverMapper.queryInfo(page, dto);
    }

    /**
     * 导入移交信息
     *
     * @param dto dto
     * @return boolean
     */
    @Override
    public boolean importTurnOver(ImportTurnOverDto dto) {
        if (dto == null) {
            throw new ToflyDeniedException("传入数据为空");
        }
        List<ImportTurnOver> turnOver = dto.getTurnOver();

        if (turnOver == null || turnOver.isEmpty()) {
            throw new ToflyDeniedException("excel上传内容为空");
        }

        if (dto.getType() == null) {
            throw new ToflyDeniedException("移交类型不能为空");
        }

        //收集水表编号
        List<String> codeList = turnOver.stream()
                .map(ImportTurnOver::getCode)
                .collect(Collectors.toList());
        //移交类型-水表退库
        if (dto.getType() == 1) {
            List<Bwrksbdatail> integers = turnOverMapper.queryTableState(codeList, "2");

            if (integers.size() != codeList.size()) {
                throw new ToflyDeniedException("移交必须为出库水表,请确认数据");
            }
        } else {
            if (dto.getType() != 4) {
                List<Bwrksbdatail> integers = turnOverMapper.queryTableState(codeList, "9");
                if (integers.size() != codeList.size()) {
                    throw new ToflyDeniedException("移交必须为已更换水表，请确认数据");
                }
            }
        }

        //查询厂家类型口径
        List<Bwrksbdatail> rksbdatails = turnOverMapper.queryTableState(codeList, null);
        //根据编号分组
        Map<String, List<Bwrksbdatail>> groupByCode = rksbdatails.stream()
                .collect(Collectors.groupingBy(Bwrksbdatail::getBwSbbh));

        Bwsbyjgl yjgl;

        List<Bwsbyjgl> result = new ArrayList<>();


        Long userId = SecurityUtils.getUserId();

        String dateFormat = DateUtil.getDateFormat(new Date(), "yyyyMMdd-");

        String checkFlag = dto.getType() == 1 ? "2" : "0";

        for (ImportTurnOver it : turnOver) {

            yjgl = new Bwsbyjgl();
            yjgl.setBwLsbh(dateFormat + System.currentTimeMillis() + "");
            if (groupByCode.containsKey(it.getCode())) {
                Bwrksbdatail tail = groupByCode.get(it.getCode()).get(0);
                yjgl.setBwSbcj(tail.getBwSbcj());
                yjgl.setBwSblx(tail.getBwSblx());
                yjgl.setBwSbkj(tail.getBwSbkj());
            }
            yjgl.setSbzsbh(it.getCode());
            yjgl.setBwYjlx(dto.getType());
            yjgl.setBwYjyy(it.getCause());
            if (dto.getType() != 1) {
                yjgl.setBwJsbm(dto.getAcceptDept());
                yjgl.setBwJsry(dto.getAcceptUser());
            }
            yjgl.setBwYjry(dto.getTurnOverUser());
            yjgl.setBwYjrq(dto.getTurnOverTime());
            yjgl.setCreateUser(userId);
            yjgl.setCheckFlag(checkFlag);

            bwsbyjglService.save(yjgl);
            result.add(yjgl);
        }


        List<String> tableCodeList = result.stream().map(Bwsbyjgl::getSbzsbh).collect(Collectors.toList());
        if (tableCodeList.size() > 0) {
            //退库
            if (dto.getType() == 1){
                turnOverMapper.updateTableState(tableCodeList,"8");
            }else {
                //报废
                turnOverMapper.updateTableState(tableCodeList,"7");
            }

        }

        return true;
    }

    /**
     * 获取不存在数据库水表编码
     *
     * @param code code
     * @return list
     */
    @Override
    public List<String> getNonEntityCode(String code, String sbzt) {
        if (StringUtils.isEmpty(code)) {
            throw new ToflyDeniedException("传入数据为空");
        }
        List<String> codeList = Arrays.stream(code.split(",")).distinct().collect(Collectors.toList());

        List<String> collect = turnOverMapper.queryTableState(codeList,sbzt)
                .stream()
                .distinct().map(Bwrksbdatail::getBwSbbh).collect(Collectors.toList());

        return codeList.stream().filter(c -> !collect.contains(c)).collect(Collectors.toList());
    }
}
